<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2023 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Delete extends Storm_Query_Abstract {

  public function __construct(Storm_Model_Loader $loader, ?string $alias = null) {
    $this->_helper = Storm_Query_Helper::newHelper($loader, $alias);
    $this->_criteria = new Storm_Query_Criteria;
  }


  public function hasOrders() : bool {
    return false;
  }


  public function promoteToQuery() : Storm_Query {
    $query = new Storm_Query($this->getHelper()->getLoader(), $this->getHelper()->getAlias());
    $query->_criteria = $this->_criteria;

    return $query;
  }


  public function deleteAll(): int
  {
    return $this->_prepareHelper()
                ->getLoader()
                ->deleteForQuery($this);
  }


  protected function _prepareHelper() : Storm_Query_Helper {
    $this->_criteria->withHelperDo($this->getHelper());

    return $this->getHelper();
  }


  /** DB management */

  public function assembleDb() : string {
    return Zend_Db_Table_Abstract::getDefaultAdapter()
      ->sqlForDelete($this->assembleFrom(), $this->assembleWhere());
  }


  public function assembleFrom() : string {
    return $this->getHelper()->getTableName();
  }


  public function assembleWhere() : string {
    $sql = new Storm_Query_Sql;
    $this->_criteria->assemble($sql);

    return $sql->implode($this->_criteria->separator());
  }
}
