<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2022 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


abstract class Storm_Join_Abstract extends Storm_Query_Abstract {
  use Storm_Query_Trait_Select, Storm_Query_Trait_Sub;

  public function inner(Storm_Join_Inner $inner) : self {
    $this->_joins [] = $inner->setParentHelper($this->getHelper());

    return $this;
  }


  public function left(Storm_Join_Left $left) : self {
    $this->_joins [] = $left->setParentHelper($this->getHelper());

    return $this;
  }


  public function right(Storm_Join_Right $right) : self {
    $this->_joins [] = $right->setParentHelper($this->getHelper());

    return $this;
  }


  public function all_select() : self {
    $this->getHelper()->setAllSelect(true);

    return $this;
  }


  protected function _countSelect() : int {
    $count = 0;

    if ($this->getHelper()->hasSelect())
      $count++;

    foreach ($this->getJoins() as $join)
      $count += $join->_countSelect();

    return $count;
  }


  protected function _selectWithAlias() : self {
    $this->getHelper()->beWithAlias();

    foreach ($this->getJoins() as $join)
      $join->_selectWithAlias();

    return $this;
  }


  /** DB management */

  public function getAssembleSelect() : Zend_Db_Table_Select {
    $select = parent::getAssembleSelect();

    $this->assembleJoins($select);

    return $select;
  }


  protected function _assembleFrom(Zend_Db_Table_Select $select) : self {
    $select->setIntegrityCheck(false);

    return parent::_assembleFrom($select);
  }


  public function assembleJoins(Zend_Db_Table_Select $select) : self {
    $this->_assembleJoin($select);

    foreach ($this->getJoins() as $join)
      $join->assembleJoins($select);

    return $this;
  }


  protected function _assembleJoin(Zend_Db_Table_Select $select) : self {
    return $this;
  }


  /** Volatile management */

  public function instances(?array $instances = []) : array {
    $instances = parent::instances($instances);

    foreach ($this->getJoins() as $join)
      $instances = $join->instances($instances);

    return $instances;
  }
}
