<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2022 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Join_Sub extends Storm_Join_Abstract implements Storm_Query_SubInterface {

  public function __construct(Storm_Model_Loader $loader, ?string $alias = null) {
    $this->_helper = Storm_Query_Helper::newJoinHelper($loader, $alias);
    $this->_criteria = new Storm_Query_Criteria;
  }


  public function setParentHelper(Storm_Query_Helper $parent_helper) : self {
    $this->_parent_helper = $parent_helper;

    return $this;
  }


  public function getParentHelper() : ?Storm_Query_Helper {
    return $this->_parent_helper;
  }


  public function on_eq(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key));

    return $this;
  }


  public function on_not_eq(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key)
                         ->beNotEqual());

    return $this;
  }


  public function on_gt(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key)
                         ->beGreater());

    return $this;
  }


  public function on_gt_eq(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key)
                         ->beGreaterEqual());

    return $this;
  }


  public function on_lt(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key)
                         ->beLesser());

    return $this;
  }


  public function on_lt_eq(string $parent_key, string $key) : self {
    $this->_addCondition(Storm_Query_Condition::newWith($this, $parent_key, $key)
                         ->beLesserEqual());

    return $this;
  }


  protected function _addCondition(Storm_Query_Condition $condition) : self {
    if ($this->_condition) {
      $this->_condition->add($condition);

      return $this;
    }

    $this->_condition = $condition;

    return $this;
  }
}
