<?php
/**
STORM is under the MIT License (MIT)

Copyright (c) 2010-2021 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


class Storm_Model_Loader_SaveOperation
{

  protected mixed $_id;
  protected STorm_Model_Loader $_loader;
  protected string $_id_field;
  protected array $_datas;
  protected Storm_Model_PersistenceStrategy_Abstract $_persistence_strategy;

  public function __construct(Storm_Model_Loader $loader)
  {
    $this->_loader = $loader;
    $this->_persistence_strategy = $loader->getPersistenceStrategy();
    $this->_id_field = $loader->getIdField();
  }

  /**
   * @return false if value is not saveable in db
   */
  public function isValueCanBeSaved(mixed $value): bool
  {
    return is_string($value)
      || is_int($value)
      || is_float($value)
      || is_bool($value)
      || (null === $value);
  }

  protected function _prepareSaveDatas(Storm_Model_Abstract $model): self
  {
    $this->_datas = array_filter($model->attributesToArray(),
                                 [$this, 'isValueCanBeSaved']);
    $this->_id = $this->_datas['id'] ?? null;

    if ('id' !== $this->_id_field)
      unset($this->_datas['id']);

    return $this;
  }

  public function save(Storm_Model_Abstract $model, bool $is_new = false): int
  {
    $this->_prepareSaveDatas($model);

    return ($is_new || $this->_isNew($model))
      ? $this->_saveNew($model, $is_new)
      : $this->_saveExisting($model);
  }

  protected function _isNew(Storm_Model_Abstract $model): bool
  {
    return $model->isNew();
  }

  protected function _saveExisting(Storm_Model_Abstract $model): int
  {
    $model->beUpdated();
    $this->_datas[$this->_id_field] = $this->_id;

    return $this->_persistence_strategy
      ->update($this->_datas, $this->_id);
  }

  protected function _saveNew(Storm_Model_Abstract $model, bool $is_new = false): int
  {
    if ($result = $this->_persistence_strategy->insert($this->_datas)) {
      $insert_id = ($is_new || $model->isFixedId())
        ? $this->_datas[$this->_id_field]
        : $this->_persistence_strategy->lastInsertId();

      $model->beSaved()->setId($insert_id);
    }

    return $result ? 1 : 0;
  }
}
