<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Model_State
{

  protected bool $_has_been_saved = false;
  protected bool $_has_been_loaded = false;
  protected bool $_has_been_deleted = false;
  protected bool $_has_been_updated = false;

  public function resetState(): self
  {
    $this->_has_been_deleted = false;
    $this->_has_been_loaded = false;
    $this->_has_been_updated = false;

    return $this;
  }


  public function existInDatabase(): bool
  {
    if ($this->_has_been_deleted)
      return false;

    return $this->_has_been_saved
      || $this->_has_been_loaded
      || $this->_has_been_updated;
  }


  public function hasBeenSaved(): bool
  {
    return $this->_has_been_saved;
  }


  public function hasBeenLoaded(): bool
  {
    return $this->_has_been_loaded;
  }


  public function hasBeenDeleted(): bool
  {
    return $this->_has_been_deleted;
  }


  public function hasBeenUpdated(): bool
  {
    return $this->_has_been_updated;
  }


  public function isFixture(): bool
  {
    return false;
  }


  public function setSaved(bool $saved): self
  {
    $this->_has_been_deleted = false;
    $this->_has_been_saved = $saved;

    return $this;
  }


  public function setLoaded(bool $loaded): self
  {
    $this->_has_been_loaded = $loaded;

    return $this;
  }


  public function setDeleted(bool $deleted): self
  {
    $this->_has_been_deleted = $deleted;

    return $this;
  }


  public function setUpdated(bool $updated): self
  {
    $this->_has_been_deleted = false;
    $this->_has_been_updated = $updated;

    return $this;
  }


  public function beFixture(): self
  {
    return $this;
  }
}
