<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2022 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Query_Sub extends Storm_Query_Abstract {
  use Storm_Query_Trait_Select, Storm_Query_Trait_Sub;

  public function __construct(Storm_Model_Loader $loader, ?string $alias = null) {
    $this->_helper = Storm_Query_Helper::newSubHelper($loader, $alias);
    $this->_criteria = new Storm_Query_Criteria;
  }


  public function cloneClause(): self
  {
    $clone = $this->cloneQuery();

    if ($this->_parent_helper)
      $clone->_parent_helper = $this->_parent_helper->cloneSubHelper();

    $clone->_is_negated = $this->_is_negated;

    return $clone;
  }


  public function withHelperDo(?Storm_Query_Helper $helper = null) : self {
    if ( ! $this->_parent_helper)
      $this->_parent_helper = $helper;

    $this->_criteria->withHelperDo($this->_helper);

    return $this;
  }


  public function setParentHelper(Storm_Query_Helper $parent_helper) : self {
    $this->_parent_helper = $parent_helper;

    return $this;
  }


  public function getParentHelper() : ?Storm_Query_Helper {
    return $this->_parent_helper;
  }


  public function beNegated() : self {
    $this->_is_negated = true;

    return $this;
  }


  public function isNegated() : bool {
    return $this->_is_negated;
  }


  protected function _prepareHelper() : Storm_Query_Helper {
    return $this->getHelper();
  }
}
