<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2023 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Query_SubCriteria {

  protected array $_criterias;
  protected bool $_criteria_or = false;

  public function __construct(array $criterias) {
    $this->_criterias = $criterias;
  }


  public function cloneSubCriteria(): self
  {
    $clone = new static(array_map(fn($clause) => $clause->cloneClause(),
                                  $this->_criterias));

    $clone->_criteria_or = $this->_criteria_or;

    return $clone;
  }


  public function hasClauses() : bool {
    foreach ($this->_criterias as $criteria)
      if ($criteria->hasClauses())
        return true;

    return false;
  }


  public function beOr() : self {
    $this->_criteria_or = true;

    return $this;
  }


  public function sql(Storm_Query_Abstract $query) : string {
    $sql = new Storm_Query_Sql;

    foreach ($this->_criterias as $alias => $criteria)
      $criteria
        ->withHelperDo($query->helperBy($alias))
        ->assemble($sql);

    $separator = $this->_criteria_or
      ? Storm_Query_Criteria::SEPARATOR_OR
      : Storm_Query_Criteria::SEPARATOR_AND;

    return '(' . $sql->implode(' ' . $separator . ' ') . ')';
  }


  public function filterWith(Storm_Query_Instances $query_instances,
                             Storm_Query_Abstract $query) : self {
    $filter = new Storm_Collection;

    $query_instances->instancesFor($query->getHelper()->getAlias())
                    ->select(fn($instance) => $this->_contains($instance))
                    ->eachDo(fn($instance) => $filter->add($instance));

    $query_instances->addAll([$query->getHelper()->getAlias()], $filter);

    return $this;
  }


  protected function _contains(Storm_Query_Instance $instance) : bool {
    $contain_all = ! $this->_criteria_or;

    foreach ($this->_criterias as $alias => $criteria) {
      $contain = $this->_containsFor($instance, $criteria, $alias);

      $contain_all = $this->_criteria_or
        ? $contain_all || $contain
        : $contain_all && $contain;
    }

    return $contain_all;
  }


  protected function _containsFor(Storm_Query_Instance $instance,
                                  Storm_Query_Criteria $criteria,
                                  string $alias) : bool {
    $raw_attributes = ($linked = $instance->findLinkedBy($alias))
      ? $linked->first()
      : [];

    return $criteria->containsAllAttributes($raw_attributes);
  }
}
