<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2022 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


trait Storm_Query_Trait_Group {

  /**
   * @param $value_or_array : string | array<string>
   */
  public function group($value_or_array) : self {
    if ( ! is_array($value_or_array))
      $value_or_array = [$value_or_array];

    foreach ($value_or_array as $value)
      $this->getHelper()->setGroupBy($value);

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_gt(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::greater($column, $value));

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_gt_eq(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::greaterEqual($column, $value));

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_eq(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::equal($column, $value));

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_not_eq(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::equal($column, $value)
                               ->setNegated(true));

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_lt(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::lesser($column, $value));

    return $this;
  }


  /**
   * @param $value int | string
   */
  public function having_lt_eq(string $column, $value) : self {
    $this->getHelper()->having($column, Storm_Query_Clause::lesserEqual($column, $value));

    return $this;
  }


  /**
   * @param $column : string | Storm_Query_MatchRating | Storm_Query_Clause_OrderBy
   */
  public function order($column) : self {
    $this->_orders [] = $this->_order($column);

    return $this;
  }


  /**
   * @param $column : string | Storm_Query_MatchRating | Storm_Query_Clause_OrderBy
   */
  public function order_desc($column) : self {
    $this->_orders [] = $this->_order($column)
      ->beDesc();

    return $this;
  }


  /**
   * @param $column : string | Storm_Query_MatchRating | Storm_Query_Clause_OrderBy
   */
  protected function _order($column) : Storm_Query_Clause {
    if ($column instanceof Storm_Query_MatchRating)
      return Storm_Query_Clause::orderMatch($column->keyOrArray(),
                                            Storm_Query_Clause::match($column));

    return ($column instanceof Storm_Query_Clause_OrderBy)
      ? $column
      : Storm_Query_Clause::order($column);
  }


  public function rand(): self
  {
    $this->_is_rand = true;

    return $this;
  }


  public function fetchAll(bool $do_cache = false) : array {
    return $this->_prepareHelper()
                ->loaderForFetch()
                ->fetchForQuery($this, $do_cache);
  }


  public function countAll() : int {
    $this->_orders = [];
    $this->_clause_limit = null;
    $this->_clause_limit_page = null;
    $this->getHelper()->addCountAll();

    return $this->_prepareHelper()
                ->loaderForFetch()
                ->countForQuery($this);
  }


  public function fetchFirst(bool $do_cache = false)
  {
    return $this->limit(1)
                ->_prepareHelper()
                ->loaderForFetch()
                ->fetchFirstForQuery($this, $do_cache);
  }


  /** DB management */

  protected function _assembleGroupBy(Zend_Db_Table_Select $select) : self {
    $this->getHelper()->assembleGroupBy($select);

    return $this;
  }


  protected function _assembleOrders(Zend_Db_Table_Select $select) : self {
    foreach ($this->_orders as $order)
      $order->assemble($select);

    if ($this->_is_rand)
      $select->order(new Zend_Db_Expr('RAND()'));

    return $this;
  }


  protected function _assembleHaving(Zend_Db_Table_Select $select) : self {
    if ( ! ($keys = $this->getHelper()->getSelect()))
      return $this;

    foreach ($keys as $key)
      $this->_assembleForKey($select, $key);

    return $this;
  }


  protected function _assembleForKey(Zend_Db_Table_Select $select,
                                     Storm_Query_Key $key) : self {
    foreach ($key->clauses() as $clause)
      $select->having($clause->getFormatDb());

    return $this;
  }


  /** Volatile management */

  public function orderVolatile(Storm_Query_Instance $a, Storm_Query_Instance $b): int
  {
    return $this->_is_rand
      ? $this->getHelper()->getLoader()->randCallbackForModel($a, $b)
      : Storm_Query_Order::parseOrderClause($this->_orders)->compareVolatile($a, $b);
  }
}
