<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2025 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/

require_once __DIR__ . '/Query/Fixtures.php';

class Storm_Test_AddDependentsTest extends Storm_Test_ModelTestCase
{

  protected array $_storm_scopes = ['AddDependentsTest'];

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function addDependentAlreadyExistsOnModelShouldNotBeAdded()
  {
    $dependent = $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                ['id' => 11,
                                 'label' => 'dependent label 11']);
    $user = $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                           ['id' => 1,
                            'label' => 'user label 1',
                            'alls' => [$dependent]]);

    $this->assertEquals(1, $user->numberOfAlls());

    $user
      ->addAll($dependent)
      ->assertSave();

    $this->assertEquals(1, $user->numberOfAlls());
  }


  /** @test */
  public function addDoubleDependentOnModelShouldNotBeAdded()
  {
    $dependent = $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                ['id' => 11,
                                 'label' => 'dependent label 11']);
    $user = $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                           ['id' => 1,
                            'label' => 'user label 1']);

    $user
      ->addAll($dependent)
      ->assertSave();

    $this->assertEquals(1, $user->numberOfAlls());

    $user
      ->addAll($dependent)
      ->assertSave();

    $this->assertEquals(1, $user->numberOfAlls());
  }


  /** @test */
  public function addDoubleDependentOnModelUniqFalseShouldBeAdded()
  {
    $dependent = $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                ['id' => 11,
                                 'label' => 'dependent label 11']);
    $user = $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                           ['id' => 1,
                            'label' => 'user label 1']);

    $user
      ->addNotunik($dependent)
      ->assertSave();

    $this->assertEquals(1, $user->numberOfNotuniks());

    $user
      ->addNotunik($dependent)
      ->assertSave();

    $this->assertEquals(2, $user->numberOfNotuniks());
  }


  /** @test */
  public function addAnotherDependentOrderOnModelAfterLoadShouldBeAdded()
  {
    $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                   ['id' => 1,
                    'label' => 'user label 1',
                    'orders' => [$this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                                ['id' => 11,
                                                 'label' => 'dependent label 11'])]]);

    Storm_Test_LoaderQueryUsersWithDependents::find(1)
      ->resetCacheAllDependents();

    $this->assertEquals([11], array_map(fn($order) => $order->getId(),
                                        Storm_Test_LoaderQueryUsersWithDependents::find(1)
                                        ->getOrders()));

    Storm_Test_LoaderQueryUsersWithDependents::find(1)
      ->addOrder($this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                ['id' => 12,
                                 'label' => 'dependent label 12']))
      ->assertSave();

    Storm_Test_LoaderQueryUsersWithDependents::find(1)
      ->resetCacheAllDependents();

    $this->assertEquals([11, 12], array_map(fn($order) => $order->getId(),
                                            Storm_Test_LoaderQueryUsersWithDependents::find(1)
                                            ->getOrders()));
  }


  /** @test */
  public function setDependentsWithDuplicatedModelShouldBeFiltered()
  {
    $dependent = $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                                ['id' => 11,
                                 'label' => 'dependent label 11']);
    $user = $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                           ['id' => 1,
                            'label' => 'user label 1']);

    $user
      ->setAlls([$dependent, $dependent])
      ->assertSave();

    $this->assertEquals(1, $user->numberOfAlls());
  }
}
