<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/


class Storm_Test_DependentsDeleteTest extends Storm_Test_ModelTestCase
{

  protected string $_first_type;
  protected array $_storm_scopes = ['Storm_Test_DependentsDeleteTest'];
  protected Storm_Test_DependentUser $_user_1;

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->_user_1 = $this->fixture(Storm_Test_DependentUser::class,
                                    ['id' => 1,
                                     'name' => 'user 1']);

    $this->_user_1->setFamillys([$this->fixture(Storm_Test_DependentFamilly::class,
                                                ['id' => 1,
                                                 'type' => 'brother']),
                                 $this->fixture(Storm_Test_DependentFamilly::class,
                                                ['id' => 2,
                                                 'type' => 'sister'])])
                  ->assertSave();

    $user_2 = $this->fixture(Storm_Test_DependentUser::class,
                             ['id' => 2,
                              'name' => 'user 2']);

    $member_1 = $this->fixture(Storm_Test_DependentMember::class,
                               ['id' => 1,
                                'parent_name' => 'user 1',
                                'locations' => [$this->fixture(Storm_Test_DependentLocation::class,
                                                               ['id' => 1,
                                                                'position' => 'here'])]]);
    $member_2 = $this->fixture(Storm_Test_DependentMember::class,
                               ['id' => 2,
                                'parent_name' => 'user 1']);
    $member_3 = $this->fixture(Storm_Test_DependentMember::class,
                               ['id' => 3,
                                'parent_name' => 'user 1']);
    $user_2->setFamillys([$this->fixture(Storm_Test_DependentFamilly::class,
                                         ['id' => 3,
                                          'type' => 'mother',
                                          'members' => [$member_1, $member_2]]),
                          $this->fixture(Storm_Test_DependentFamilly::class,
                                         ['id' => 4,
                                          'type' => 'father',
                                          'members' => [$member_3]])])
           ->setProfessions([$this->fixture(Storm_Test_DependentProfession::class,
                                            ['id' => 1,
                                             'label' => 'teacher'])])
           ->assertSave();

    $this->fixture(Storm_Test_DependentOther::class,
                   ['id' => 1,
                    'familly_id' => 3]);
    $this->fixture(Storm_Test_DependentOther::class,
                   ['id' => 2,
                    'familly_id' => 1]);

    $this->_first_type = Storm_Test_DependentUser::find(1)->getFamillys()[0]->getType();
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function afterDelteWithBelongsToShouldUpdateParentHasMany()
  {
    Storm_Test_DependentFamilly::find(1)->delete();
    $famillys = Storm_Test_DependentUser::find(1)->getFamillys();
    $this->assertNotEquals($this->_first_type, $famillys[0]->getType());
    $this->assertCount(1, $famillys);
  }


  /** @test */
  public function afterBasicDeleteWithBelongsToShouldUpdateParentHasMany()
  {
    Storm_Test_DependentFamilly::basicDeleteBy(['id' => 1]);
    $famillys = Storm_Test_DependentUser::find(1)->getFamillys();
    $this->assertNotEquals($this->_first_type, $famillys[0]->getType());
    $this->assertCount(1, $famillys);
  }


  /** @test */
  public function deleteQueryUser2ShouldDeleteAllDependentsOptimized()
  {
    Storm_Test_DependentUser::deleteQuery()
      ->eq('name', 'user 2')
      ->deleteAll();

    $this->assertSqlEquals(["SELECT `famillys`.* FROM `famillys` WHERE (`famillys`.`id_user` = 1 AND `famillys`.`id` NOT IN (1, 2))",
                            "SELECT `locations`.* FROM `locations` WHERE (`locations`.`id_member` = 1 AND `locations`.`id` != 1)",
                            "SELECT `members`.* FROM `members` WHERE (`members`.`id_familly` = 3 AND `members`.`id` NOT IN (1, 2))",
                            "SELECT `members`.* FROM `members` WHERE (`members`.`id_familly` = 4 AND `members`.`id` != 3)",
                            "SELECT `famillys`.* FROM `famillys` WHERE (`famillys`.`id_user` = 2 AND `famillys`.`id` NOT IN (3, 4))",
                            "SELECT `professions`.* FROM `professions` WHERE (`professions`.`id_user` = 2 AND `professions`.`id_prof` != 1)",
                            "SELECT `users`.`id` FROM `users` WHERE (`users`.`name` = 'user 2') LIMIT 1000",
                            "DELETE FROM `professions` WHERE (`professions`.`id_user` = 2)",
                            "SELECT `famillys`.`id` FROM `famillys` WHERE (`famillys`.`id_user` = 2) LIMIT 1000",
                            "DELETE FROM `others` WHERE (`others`.`familly_id` IN (3, 4))",
                            "DELETE FROM `members` WHERE (`members`.`id_familly` IN (3, 4))",
                            "DELETE FROM `famillys` WHERE (`famillys`.`id_user` = 2)",
                            "DELETE FROM `users` WHERE (`users`.`name` = 'user 2')"]);
    $this->assertNull(Storm_Test_DependentUser::find(2));
    $this->assertNull(Storm_Test_DependentFamilly::find(3));
    $this->assertNull(Storm_Test_DependentFamilly::find(4));
    $this->assertNull(Storm_Test_DependentMember::find(1));
    $this->assertNull(Storm_Test_DependentMember::find(2));
    $this->assertNull(Storm_Test_DependentMember::find(3));
    $this->assertNull(Storm_Test_DependentProfession::find(1));
    $this->assertNotNull(Storm_Test_DependentLocation::find(1));
    $this->assertNull(Storm_Test_DependentOther::find(1));
    $this->assertNotNull(Storm_Test_DependentOther::find(2));
  }


  /** @test */
  public function updateHasManyAndOnlySaveOnParentShouldUpdateSqlFamilly()
  {
    Storm_Test_DependentFamilly::find(1)->setType('changed name');
    $this->_user_1->save();
    $this->assertEquals('changed name', Storm_Test_DependentFamilly::find(1)->getType());
  }


  /** @test */
  public function updateHasManyWithFindFirstByAndOnlySaveOnParentShouldUpdateSqlFamilly()
  {
    $familly = Storm_Test_DependentFamilly::findFirstBy(['type' => 'brother']);
    $familly->setType('changed name');
    $this->_user_1->save();
    $this->assertEquals('changed name', Storm_Test_DependentFamilly::find(1)->getType());
  }
}




class Storm_Test_DependentUser extends Storm_Model_Abstract
{

  protected string $_table_name = 'users';
  protected array $_has_many = ['professions' => ['model' => Storm_Test_DependentProfession::class,
                                                  'role' => 'user',
                                                  'dependents' => 'delete'],

                                'famillys' => ['model' => Storm_Test_DependentFamilly::class,
                                               'role' => 'user',
                                               'dependents' => 'delete']];
}




class DependentFamillyLoader extends Storm_Model_Loader
{

  protected function _cascadeDelete(array $ids): self
  {
    if ($ids)
      Storm_Test_DependentOther::deleteQuery()
        ->in('familly_id', $ids)
        ->beBasic()
        ->deleteAll();

    return $this;
  }
}




class Storm_Test_DependentFamilly extends Storm_Model_Abstract
{

  protected string $_table_name = 'famillys';
  protected string $_loader_class = DependentFamillyLoader::class;
  protected array $_belongs_to = ['user' => ['model' => Storm_Test_DependentUser::class,
                                             'referenced_in' => 'id_user']];
  protected array $_has_many = ['members' => ['model' => Storm_Test_DependentMember::class,
                                              'role' => 'familly',
                                              'dependents' => 'delete']];
}




class Storm_Test_DependentMember extends Storm_Model_Abstract
{

  protected string $_table_name = 'members';
  protected array $_belongs_to = ['familly' => ['model' => Storm_Test_DependentFamilly::class,
                                                'referenced_in' => 'id_familly']];
  protected array $_has_many = ['locations' => ['model' => Storm_Test_DependentLocation::class,
                                                'role' => 'member']];
}




class Storm_Test_DependentProfession extends Storm_Model_Abstract
{

  protected string $_table_name = 'professions';
  protected string $_table_primary = 'id_prof';
  protected array $_belongs_to = ['user' => ['model' => Storm_Test_DependentUser::class,
                                             'referenced_in' => 'id_user']];
}




class Storm_Test_DependentLocation extends Storm_Model_Abstract
{

  protected string $_table_name = 'locations';
  protected array $_belongs_to = ['member' => ['model' => Storm_Test_DependentMember::class,
                                               'referenced_in' => 'id_member']];
}




class Storm_Test_DependentOther extends Storm_Model_Abstract
{

  protected string $_table_name = 'others';
}
