<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2015 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

class Storm_Test_FixedIdVolatileTest extends Storm_Test_ModelTestCase {
  protected $_model;

  public function setUp(): void
  {
    parent::setUp();

    Storm_Test_FixedIdModel::beVolatile();

    $this->_model = Storm_Test_FixedIdModel::newInstance()
      ->setCode('FIXED_CODE');
    $this->_model->save();
  }


  /** @test */
  public function shouldBeFoundWithFixedId() {
    $this->assertNotNull(Storm_Test_FixedIdModel::find('FIXED_CODE'));
  }


  /** @test */
  public function modelIdShouldBeFixed() {
    $this->assertEquals('FIXED_CODE', $this->_model->getId());
  }
}


class Storm_Test_FixedIdDbTest extends Storm_Test_ModelTestCase {
  protected $_model, $_table;

  public function setUp(): void
  {
    parent::setUp();

    $this->_table = $this->mock()
                         ->whenCalled('insert')
                         ->answers(1);

    Storm_Test_FixedIdModel::setTable($this->_table);

    $this->_model = Storm_Test_FixedIdModel::newInstance()
      ->setCode('FIXED_CODE');

    $this->_model->save();
  }


  /** @test */
  public function shouldCallInsertWithFixedId() {
    $this->assertTrue($this->_table
                      ->methodHasBeenCalledWithParams('insert',
                                                      [ ['code' => 'FIXED_CODE'] ]));
  }
}



class Storm_Test_FixedIdModel extends Storm_Model_Abstract
{

  protected string $_table_primary = 'code';
  protected bool $_fixed_id = true;
}
