<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2025 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/


class Storm_Test_InstanceofDependentTest extends Storm_Test_ModelTestCase
{

  protected StormTestInstanceofUsers $_users_1;
  protected StormTestInstanceofUsers $_users_3;

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->_users_1 = $this->fixture(StormTestInstanceofUsers::class,
                                     ['id' => 1,
                                      'label' => 'User 1']);
    $this->_users_3 = $this->fixture(StormTestInstanceofUsers::class,
                                     ['id' => 3,
                                      'label' => 'User 3']);

    $this->fixture(StormTestInstanceofDependents::class,
                   ['id' => 1,
                    'id_user' => 1,
                    'label' => 'Depends 1']);
    $this->fixture(StormTestInstanceofDependents::class,
                   ['id' => 2,
                    'id_user' => 1,
                    'label' => 'Depends 2']);
    $this->fixture(StormTestInstanceofDependents::class,
                   ['id' => 3,
                    'id_user' => 2,
                    'label' => 'Depends 3']);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function getDependsForUser_1_ShouldReturnStormCollectionWithAllDependents()
  {
    $this->assertTrue($this->_users_1->getDepends() instanceof Storm_Collection);
    $this->assertEquals(['Depends 1', 'Depends 2'],
                        $this->_users_1->getDepends()
                        ->collect(fn($depends) => $depends->getLabel())
                        ->getArrayCopy());
  }


  /** @test */
  public function getDependsForUser_3_ShouldReturnEmptyStormCollection()
  {
    $this->assertTrue($this->_users_3->getDepends() instanceof Storm_Collection);
    $this->assertTrue($this->_users_3->getDepends()->isEmpty());
  }
}




class StormTestInstanceofUsers extends Storm_Model_Abstract
{

  protected string $_table_name = 'users';
  protected array $_has_many =
    ['depends' => ['model' => StormTestInstanceofDependents::class,
                   'role' => 'user',
                   'instance_of' => Storm_Collection::class],
    ];
}




class StormTestInstanceofDependents extends Storm_Model_Abstract
{

  protected string $_table_name = 'dependents';
  protected array $_belongs_to =
    ['user' => ['model' => StormTestInstanceofUsers::class,
                'referenced_in' => 'id_user']];
}
