<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2025 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/

require_once __DIR__ . '/Query/Fixtures.php';

class Storm_Test_LoadDependentsTest extends Storm_Test_ModelTestCase
{

  protected array $_storm_scopes = ['LoadDependentsTest'];
  protected Storm_Test_LoaderQueryUsersWithDependents $_user;

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->_user = $this->fixture(Storm_Test_LoaderQueryUsersWithDependents::class,
                                  ['id' => 1,
                                   'label' => 'User name 1']);

    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 1,
                    'label' => 'other name',
                    'age' => 20,
                    'with_scope' => 0,
                    'without_scope' => 0,
                    'id_user' => 1]);
    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 2,
                    'label' => 'name',
                    'age' => 10,
                    'with_scope' => 1,
                    'without_scope' => 1,
                    'id_user' => 1]);
    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 3,
                    'label' => 'name',
                    'age' => 30,
                    'with_scope' => 1,
                    'without_scope' => 0,
                    'id_user' => 1]);
    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 4,
                    'label' => 'other name',
                    'age' => 15,
                    'with_scope' => 2,
                    'without_scope' => 0,
                    'id_user' => 1]);
    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 6,
                    'label' => 'other name',
                    'age' => 10,
                    'with_scope' => 1,
                    'without_scope' => 0,
                    'id_user' => 1]);
    $this->fixture(Storm_Test_LoaderQueryDependentsForUsers::class,
                   ['id' => 5,
                    'label' => 'name',
                    'age' => 20,
                    'with_scope' => 1,
                    'without_scope' => 0,
                    'id_user' => 1]);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function dependentsAllsShouldHave_6_Models()
  {
    $this->assertEquals(6, $this->_user->numberOfAlls());
    $this->assertEquals([1, 2, 3, 4, 6, 5], array_map(fn($all) => $all->getId(),
                                                      $this->_user->getAlls()));
    $this->assertSqlEquals(["SELECT COUNT(*) AS `numberof` FROM `dependents` WHERE (`dependents`.`id_user` = 1)",
                            "SELECT `dependents`.* FROM `dependents` WHERE (`dependents`.`id_user` = 1 AND `dependents`.`id` NOT IN (1, 2, 3, 4, 6, 5))"]);
  }


  /** @test */
  public function dependentsOrdersShouldHave_2_Models()
  {
    $this->assertEquals(6, $this->_user->numberOfOrders());
    $this->assertEquals([3, 5], array_map(fn($all) => $all->getId(),
                                          $this->_user->getOrders()));
    $this->assertSqlEquals(["SELECT COUNT(*) AS `numberof` FROM `dependents` WHERE (`dependents`.`id_user` = 1)",
                            "SELECT `dependents`.* FROM `dependents` WHERE (`dependents`.`id_user` = 1) ORDER BY `dependents`.`label` ASC, `dependents`.`age` DESC LIMIT 2"]);
  }


  /** @test */
  public function dependentsScopesShouldHave_2_Models()
  {
    $this->assertEquals(3, $this->_user->numberOfScopes());
    $this->assertEquals([3, 6], array_map(fn($all) => $all->getId(),
                                          $this->_user->getScopes()));
    $this->assertSqlEquals(["SELECT COUNT(*) AS `numberof` FROM `dependents` WHERE (`dependents`.`id_user` = 1 AND `dependents`.`with_scope` = 1 AND `dependents`.`without_scope` = 0)",
                            "SELECT `dependents`.* FROM `dependents` WHERE (`dependents`.`id_user` = 1 AND `dependents`.`with_scope` = 1 AND `dependents`.`without_scope` = 0) LIMIT 2"]);
  }


  /** @test */
  public function dependentsClausesOrdersShouldHave_6_Models()
  {
    $this->assertEquals(6, $this->_user->numberOfClausesOrders());
    $this->assertEquals([3, 5, 2, 1, 4, 6], array_map(fn($all) => $all->getId(),
                                                      $this->_user->getClausesOrders()));
    $this->assertSqlEquals(["SELECT COUNT(*) AS `numberof` FROM `dependents` WHERE (`dependents`.`id_user` = 1)",
                            "SELECT `dependents`.* FROM `dependents` WHERE (`dependents`.`id_user` = 1) ORDER BY `dependents`.`label` ASC, `dependents`.`age` DESC"]);
  }


  /** @test */
  public function dependentsClausesScopessShouldHave_4_Models()
  {
    $this->assertEquals(4, $this->_user->numberOfClausesScopes());
    $this->assertEquals([3, 4, 6, 5], array_map(fn($all) => $all->getId(),
                                                $this->_user->getClausesScopes()));
    $this->assertSqlEquals(["SELECT COUNT(*) AS `numberof` FROM `dependents` WHERE (`dependents`.`id_user` = 1 AND `dependents`.`with_scope` IN (1, 2) AND `dependents`.`without_scope` = 0)",
                            "SELECT `dependents`.* FROM `dependents` WHERE (`dependents`.`id_user` = 1 AND `dependents`.`with_scope` IN (1, 2) AND `dependents`.`without_scope` = 0 AND `dependents`.`id` NOT IN (3, 4, 6, 5))"]);
  }
}
