<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

class Storm_Test_Mock_User extends Storm_Model_Abstract {

  protected string $_table_name = 'users';
  protected string $_loader_class = 'Storm_Test_Mock_UserLoader';
  protected $_valid = true;
  protected $_trace_hook_mock;
  protected array $_default_attribute_values = ['name' => 'Hari',
                                                'first_name' => 'Mata',
                                                'like_cats' => true];
  protected array $_has_many = ['animals' => []];

  public function descriptionOfAnimals() {
    return ($this->getLikeCats()
            ? ['model' => 'Storm_Test_Mock_Cat',
               'role' => 'owner' ]
            : ['model' => 'Storm_Test_Mock_Dog',
               'role' => 'owner',
               'limit' => 5]);
  }


  public function setValid($valid) {
    $this->_valid = $valid;
    return $this;
  }


  public function isValid(): bool
  {
    return $this->_valid;
  }


  public function setTraceHookMock($mock) {
    $this->_trace_hook_mock = $mock;
    return $this;
  }


  public function beforeSave(): self
  {
    if ($this->_trace_hook_mock !== null)
      $this->_trace_hook_mock->beforeSave();

    return $this;
  }


  public function afterSave(): self
  {
    if ($this->_trace_hook_mock !== null)
      $this->_trace_hook_mock->afterSave();

    return $this;
  }


  public function loaderSave() {
    if ($this->_trace_hook_mock !== null)
      return $this->_trace_hook_mock->loaderSave();
    return null;
  }


  public function afterDelete(): self
  {
    if ($this->_trace_hook_mock !== null)
      $this->_trace_hook_mock->afterDelete();

    return $this;
  }


  public function beforeDelete(): self
  {
    if ($this->_trace_hook_mock !== null)
      $this->_trace_hook_mock->beforeDelete();

    return $this;
  }
}
