<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/


class Storm_Test_ModelWithParentModelTest extends Storm_Test_ModelTestCase
{

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->fixture(Storm_Test_ModelWithParentAnimal::class,
                   ['id' => 1,
                    'label' => 'carnivorous']);

    $this->fixture(Storm_Test_ModelWithParentDog::class,
                   ['id' => 2,
                    'label' => 'greyhound']);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    parent::tearDown();
  }


  /** @test */
  public function findAnimal_1_ShouldReturnModel()
  {
    $animal = Storm_Test_ModelWithParentAnimal::find(1);
    $this->assertEquals('carnivorous', $animal->getLabel());
    $this->assertTrue( ! ($animal instanceof Storm_Test_ModelWithParentDog));
  }


  /** @test */
  public function findAnimal_2_ShouldReturnModel()
  {
    $animal = Storm_Test_ModelWithParentAnimal::find(2);
    $this->assertEquals('greyhound', Storm_Test_ModelWithParentAnimal::find(2)->getLabel());
    $this->assertTrue($animal instanceof Storm_Test_ModelWithParentDog);
  }


  /** @test */
  public function findDog_1_ShouldReturnModel()
  {
    $dog = Storm_Test_ModelWithParentDog::find(1);
    $this->assertEquals('carnivorous', $dog->getLabel());
    $this->assertTrue(! ($dog instanceof Storm_Test_ModelWithParentDog));
  }


  /** @test */
  public function findDog_2_ShouldReturnModel()
  {
    $dog = Storm_Test_ModelWithParentDog::find(2);
    $this->assertEquals('greyhound', $dog->getLabel());
    $this->assertTrue($dog instanceof Storm_Test_ModelWithParentDog);
  }
}




class Storm_Test_ModelWithParentAnimal extends Storm_Model_Abstract
{

  protected $_table_name = 'animals';
}




class Storm_Test_ModelWithParentDog extends Storm_Test_ModelWithParentAnimal
{

  protected static ?string $_class_name_parent = Storm_Test_ModelWithParentAnimal::class;
}
