<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/


class Storm_Test_Query_FindAllByCacheTest extends Storm_Test_ModelTestCase
{

  protected array $_storm_scopes = [Storm_Test_Query_FindAllByCacheTest::class];

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->fixture(FindAllByCache_Users::class,
                   ['id' => 1,
                    'label' => 'User name 1',
                    'age' => 10]);
    $this->fixture(FindAllByCache_Users::class,
                   ['id' => 2,
                    'label' => 'User name 2',
                    'age' => 20]);
    $this->fixture(FindAllByCache_Users::class,
                   ['id' => 3,
                    'label' => 'User name 3',
                    'age' => 30]);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function withNoCacheShouldNotifySql_3_Times()
  {
    FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll();
    FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll();
    $models = FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll();

    $this->assertSqlCount("SELECT `users`.* FROM `users` WHERE (`users`.`age` > 15)", 3);
    $this->assertEquals(['User name 2', 'User name 3'],
                        array_map(fn($model) => $model->getLabel(),
                                  $models));
  }


  /** @test */
  public function withCacheShouldNotifySqlOnly_1_Time()
  {
    $query = FindAllByCache_Users::query()
      ->gt('age', 15);

    $query->fetchAll(true);
    $query->fetchAll(true);
    $models = $query->fetchAll(true);

    $this->assertSqlCount("SELECT `users`.* FROM `users` WHERE (`users`.`age` > 15 AND `users`.`id` NOT IN (2, 3))", 1);
    $this->assertEquals(['User name 2', 'User name 3'],
                        array_map(fn($model) => $model->getLabel(),
                                  $models));
  }


  /** @test */
  public function withClearCacheShouldNotify_2_Times()
  {
    FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll(true);
    FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll(true);
    FindAllByCache_Users::clearCache();
    $models = FindAllByCache_Users::query()
      ->gt('age', 15)
      ->fetchAll(true);

    $this->assertSqlCount("SELECT `users`.* FROM `users` WHERE (`users`.`age` > 15 AND `users`.`id` NOT IN (2, 3))", 2);
    $this->assertEquals(['User name 2', 'User name 3'],
                        array_map(fn($model) => $model->getLabel(),
                                  $models));
  }
}




class FindAllByCache_Users extends Storm_Model_Abstract
{

  protected $_table_name = 'users';
}
