<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/


class Storm_Test_LoaderQueryUser extends Storm_Model_Abstract
{

  protected string $_table_name = 'users';
}




class Storm_Test_LoaderQueryMovie extends Storm_Model_Abstract
{

  protected string $_table_name = 'movies';
}




class Storm_Test_LoaderQueryActor extends Storm_Model_Abstract
{

  protected string $_table_name = 'actors';
}




class Storm_Test_LoaderQueryProductor extends Storm_Model_Abstract
{

  protected string $_table_name = 'productors';
}




class Storm_Test_LoaderQueryUsersWithDependents extends Storm_Model_Abstract
{

  protected string $_table_name = 'users_dependents';
  protected array $_has_many =
    ['alls' => ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                'role' => 'user'],
     'notuniks' => ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                    'role' => 'user',
                    'unique' => false],
     'orders' => ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                  'role' => 'user',
                  'order' => ' label  ,  age  DESC ',
                  'limit' => 2],
     'scopes' => ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                  'role' => 'user',
                  'scope' => ['with_scope' => 1,
                              'without_scope' => 0],
                  'limit' => 2],
    ];

  public function __construct()
  {
    $this->_has_many['clauses_orders'] = ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                                          'role' => 'user',
                                          'order' => [Storm_Query_Clause::order('label'),
                                                      Storm_Query_Clause::order('age')->beDesc()]];
    $this->_has_many['clauses_scopes'] = ['model' => Storm_Test_LoaderQueryDependentsForUsers::class,
                                          'role' => 'user',
                                          'scope' => [Storm_Query_Clause::in('with_scope', [1, 2]),
                                                      Storm_Query_Clause::equal('without_scope', 0)]];

    parent::__construct();
  }
}




class Storm_Test_LoaderQueryDependentsForUsers extends Storm_Model_Abstract
{

  protected string $_table_name = 'dependents';
  protected array $_belongs_to =
    ['user' => ['model' => Storm_Test_LoaderQueryUsersWithDependents::class,
                'referenced_in' => 'id_user']];
}
