<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/

require_once __DIR__ . '/Fixtures.php';

class Storm_Test_Query_LoaderDeleteTest extends Storm_Test_ModelTestCase
{

  protected array $_storm_scopes = ['LoaderDeleteTest'];

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->fixture(Storm_Test_LoaderQueryUser::class,
                   ['id' => 1,
                    'name' => 'User name 1',
                    'age' => 21]);

    $this->fixture(Storm_Test_LoaderQueryUser::class,
                   ['id' => 2,
                    'name' => 'User name 2',
                    'age' => 22]);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function onDeleteBasicShouldReturnDeleteQueryAndDeleteVolatile()
  {
    $count = Storm_Test_LoaderQueryUser::deleteQuery()
      ->beBasic()
      ->match((new Storm_Query_MatchBoolean('age'))
              ->against_or(22))
      ->deleteAll();

    $this->assertEquals(1, $count);
    $this->assertEquals([['id' => 1,
                          'name' => 'User name 1',
                          'age' => 21]],
                        array_map(fn($user) => $user->attributesToArray(),
                                  Storm_Test_LoaderQueryUser::findAllBy([])));
    $this->assertSqlEquals(["DELETE FROM `users` WHERE (MATCH(`users`.`age`) AGAINST('22' IN BOOLEAN MODE))"]);
  }


  /** @test */
  public function onDeleteNonBasicShouldReturnSelectQueryAndDeleteVolatile()
  {
    $count = Storm_Test_LoaderQueryUser::deleteQuery()
      ->match((new Storm_Query_MatchBoolean('age'))
              ->against_or(22))
      ->deleteAll();

    $this->assertEquals(1, $count);
    $this->assertEquals([['id' => 1,
                          'name' => 'User name 1',
                          'age' => 21]],
                        array_map(fn($user) => $user->attributesToArray(),
                                  Storm_Test_LoaderQueryUser::findAllBy([])));
    $this->assertSqlEquals(["DELETE FROM `users` WHERE (MATCH(`users`.`age`) AGAINST('22' IN BOOLEAN MODE))"]);
  }


  /** @test */
  public function onDeleteAllUsersMoreThan100NonBasicShouldReturnSelectQueryAndDeleteVolatile()
  {
    foreach (range(100, 200) as $id)
      $this->fixture(Storm_Test_LoaderQueryUser::class,
                     ['id' => $id,
                      'name' => 'User name ' . $id,
                      'age' => $id]);

    $this->assertEquals(103, Storm_Test_LoaderQueryUser::countBy([]));
    $count = Storm_Test_LoaderQueryUser::deleteQuery()
      ->deleteAll();

    $this->assertSqlCount("DELETE FROM `users`", 1);
    $this->assertEquals(103, $count);
  }
}
