<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2024 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/

require_once __DIR__ . '/Fixtures.php';

class Storm_Test_Query_LoaderWithErrorsTest extends Storm_Test_ModelTestCase
{

  protected array $_storm_scopes = ['LoaderWithErrorsTest'];

  public function setUp(): void
  {
    parent::setUp();

    Storm_Model_Loader::defaultToVolatile();

    $this->fixture(Storm_Test_LoaderQueryUser::class,
                   ['id' => 1,
                    'name' => 'User name 1',
                    'actor_id' => 1,
                    'age' => 21]);

    $this->fixture(Storm_Test_LoaderQueryActor::class,
                   ['id' => 2,
                    'id_actor' => 1,
                    'label' => 'Actor 2',
                    'age' => 22]);
  }


  public function tearDown(): void
  {
    Storm_Model_Loader::defaultToDb();
    Storm_Model_Loader::resetCache();
    parent::tearDown();
  }


  /** @test */
  public function existsWithNoConditionAndSelectColumnReturnNothingAndSqlContainErrorMessage()
  {
    try {
      Storm_Test_LoaderQueryUser::query()
        ->exists(Storm_Test_LoaderQueryActor::subExists()
                 ->select(['error']))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For Exists don\'t add select column, Error: For Exists condition "on" is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function inWithNoSelectColumnReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::query()
        ->in('actor_id', Storm_Test_LoaderQueryActor::subIn())
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For In add select column is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function inWithMultipleSelectColumnReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::query()
        ->in('actor_id', Storm_Test_LoaderQueryActor::subIn()
             ->select(['id', 'id_actor']))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For In add select must have only one column');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function innerWithNoConditionReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::join('tab1')
        ->all_select()
        ->inner(Storm_Test_LoaderQueryActor::inner('tab2'))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For Inner condition "on" is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function innerWithNoSelectReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::join('tab1')
        ->inner(Storm_Test_LoaderQueryActor::inner('tab2')
                ->on_eq('actor_id', 'id_actor'))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For Join a selection is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function leftWithNoConditionReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::join('tab1')
        ->all_select()
        ->left(Storm_Test_LoaderQueryActor::left('tab2'))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For Left condition "on" is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function rightWithNoConditionReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::join('tab1')
        ->all_select()
        ->right(Storm_Test_LoaderQueryActor::right('tab2'))
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: For Right condition "on" is mandatory');
      return;
    }

    $this->assertTrue(false);
  }


  /** @test */
  public function withRandAndOrderShouldReturnNothingAndSqlContainErrorMessage()
  {
    try {
      $results = Storm_Test_LoaderQueryUser::query()
        ->rand()
        ->order('name')
        ->fetchAll();
    } catch (Exception $e) {
      $this->assertSql('Error: You can\'t have rand order with classic clause order');
      return;
    }

    $this->assertTrue(false);
  }
}
